//////////////////////////////////////////////////////////////////////////////
// Monte Carlo
//
// Last update: 13/06/2010
//////////////////////////////////////////////////////////////////////////////
import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;


import jv.function.PuFunction;
import jv.geom.PgElementSet;
import jv.geom.PgPointSet;
import jv.geom.PgPolygonSet;
import jv.project.PvLightIf;
import jv.vecmath.PdVector;
import jv.vecmath.PiVector;
import jv.viewer.PvDisplay;
import jv.project.PvCameraIf;
import jv.viewer.PvViewer;
import jvx.surface.PgDomainDescr;
import jvx.surface.PgParmSurface;

import netscape.javascript.JSObject;
import netscape.javascript.JSException;


public class MonteCarlo extends Applet
{
   PgPointSet sop;
   PgParmSurface sphere;
   
   private JSObject window;
   private JSObject document;
   private boolean isLiveConnectOK = true;

   PvDisplay disp;
   protected PvCameraIf cam;      
   
   protected KeyListener internalKeyListener;
   
   protected double fieldOfView = 0.5;
   protected double sphereTransparency = 0.9;
   
	
   // This procedure is executed when the applet starts
   public void init()
   {
      // Viewer
      PvViewer viewer = new PvViewer(this, null);

      // Display
      disp = new PvDisplay();
      disp.setBackgroundColor(Color.WHITE);
      disp.showDepthcue(true);
      disp.setEnabledZBuffer(true); 
      // disp.setLightingModel(PvLightIf.LIGHT_AMBIENT); 

      // Cube
      PgPolygonSet cube;
      cube = new PgPolygonSet(3); // This is a 3D object
      cube.setName("Cube");
      cube.setNumVertices(8); // m_curve has 8 vertices
      cube.setNumPolygons(6);
      cube.setGlobalPolygonColor(Color.RED);

      // Defining the vertices coordinates
      cube.setVertex(0, 0, 0, 0);
      cube.setVertex(1, 1, 0, 0);
      cube.setVertex(2, 1, 1, 0);
      cube.setVertex(3, 0, 1, 0);
      cube.setVertex(4, 0, 0, 1);
      cube.setVertex(5, 1, 0, 1);
      cube.setVertex(6, 1, 1, 1);
      cube.setVertex(7, 0, 1, 1);
      
      PiVector idx01 = new PiVector(5);
      idx01.setEntry(0, 0);
      idx01.setEntry(1, 1);
      idx01.setEntry(2, 2);
      idx01.setEntry(3, 3);
      idx01.setEntry(4, 0);      
      cube.setPolygon(0, idx01);

      PiVector idx02 = new PiVector(5);
      idx02.setEntry(0, 4);
      idx02.setEntry(1, 5);
      idx02.setEntry(2, 6);
      idx02.setEntry(3, 7);
      idx02.setEntry(4, 4);      
      cube.setPolygon(1, idx02);
     
      PiVector idx03 = new PiVector(2);
      idx03.setEntry(0, 0);
      idx03.setEntry(1, 4);
      cube.setPolygon(2, idx03);
      
      PiVector idx04 = new PiVector(2);
      idx04.setEntry(0, 1);
      idx04.setEntry(1, 5);
      cube.setPolygon(3, idx04);
      
      PiVector idx05 = new PiVector(2);
      idx05.setEntry(0, 2);
      idx05.setEntry(1, 6);
      cube.setPolygon(4, idx05);
      
      PiVector idx06 = new PiVector(2);
      idx06.setEntry(0, 3);
      idx06.setEntry(1, 7);
      cube.setPolygon(5, idx06);
      
      cube.showVertices(false);
      cube.setPolygonSize(0, 0.1);
      cube.setPolygonSize(1, 0.1);
      cube.setPolygonSize(2, 0.1);
      cube.setPolygonSize(3, 0.1);
      cube.setPolygonSize(4, 0.1);
      cube.setPolygonSize(5, 0.1);            
      cube.showPolygonSizes(true);
           
      // Sphere
      // The platonic circumsphere
      // Setup globe as a parametrized surface
      PuFunction m_function_c = new PuFunction(2, 3);
      m_function_c.setName("Globe");
      m_function_c.setExpression(0, "0.5*cos(v)*cos(u) + 0.5");
      m_function_c.setExpression(1, "0.5*cos(v)*sin(u) + 0.5");
      m_function_c.setExpression(2, "0.5*sin(v) + 0.5");

      // Setup parametrized surface domain discretization
      PgDomainDescr m_domain_c = new PgDomainDescr(2);
      m_domain_c.setName("Globe Parametrization Domain");
      m_domain_c.setMaxSize(-Math.PI, -Math.PI/2, Math.PI, Math.PI/2);
      m_domain_c.setSize(-Math.PI, -Math.PI/2, Math.PI, Math.PI/2);
      m_domain_c.setDiscr(100, 50);
      m_domain_c.init();

      // Computes parametrized surface
      sphere = new PgParmSurface(3);
      sphere.setDomainDescr(m_domain_c);
      sphere.setFunctionExpr(m_function_c);
      sphere.compute();

      // Setup parametrized surface appearance
      sphere.setGlobalElementColor(Color.RED);
      sphere.showEdges(false);
      sphere.setTransparency(0.9);
      sphere.showTransparency(true); 
      
      // Set of points
      sop = new PgPointSet(3);
      sop.setGlobalVertexColor(Color.BLUE);
      sop.setGlobalVertexSize(0.5);
      sop.showVertexSizes(true);
      
      // Register our cube in the display
      disp.addGeometry(cube);
      disp.addGeometry(sphere);
      disp.addGeometry(sop);
      
      // Applet layout, quite simple now
      setLayout(new BorderLayout());
      add(disp, BorderLayout.CENTER);
      
      cam = disp.getCamera();
      
      // JavaScript Live Connect
      // Get references to HTML textfields via JavaScript        
      try
      {
         window = (JSObject) JSObject.getWindow(this); // this = applet
      } 
      catch(JSException jse) 
      {
         jse.printStackTrace();
         System.out.println(jse.getWrappedException().toString());           
         isLiveConnectOK = false;
         System.out.println("JSException ...");
      } 
      catch (Exception e) 
      {
         e.printStackTrace();
      }
       
      if (window != null)
      {
          window.call("ggbOnInit", null);
      }
	  else
	  {
    	  System.out.println("LCG: unable to connect to browser!");
      }        
      
      // Key events
      internalKeyListener = new KeyAdapter() 
      {
          public void keyPressed(KeyEvent keyEvent) 
          {
        	  
        	  if (keyEvent.getKeyCode() == KeyEvent.VK_1)
        	  {
        		  sphereTransparency -= 0.05;
        		  if (sphereTransparency <= 0.0)
        		  {
        			  sphereTransparency = 0.0;
        		  };
        		  sphere.setTransparency(sphereTransparency);
        		  sphere.update(sphere);
        	  };        	  
        	  
        	  if (keyEvent.getKeyCode() == KeyEvent.VK_2)
        	  {
        		  sphereTransparency += 0.05;
        		  if (sphereTransparency >= 1.0)
        		  {
        			  sphereTransparency = 1.0;
        		  };
        		  sphere.setTransparency(sphereTransparency);
        		  sphere.update(sphere);
        	  };        	  

        	  
        	  
        	  if (keyEvent.getKeyCode() == KeyEvent.VK_3)
        	  {
        		  fieldOfView -= 0.05;
        		  if (fieldOfView <= 0.0)
        		  {
        			  fieldOfView = 0.0;
        		  };
        		  cam.setFieldOfView(fieldOfView);
        		  cam.update(cam);
        	  };        	  
        	  
        	  if (keyEvent.getKeyCode() == KeyEvent.VK_4)
        	  {
        		  fieldOfView += 0.05;
        		  if (fieldOfView >= 2.0)
        		  {
        			  fieldOfView = 2.0;
        		  };
        		  cam.setFieldOfView(fieldOfView);
        		  cam.update(cam);
        	  };        	  
        	          	          	  
        	  
        	  
        	  if ((keyEvent.getKeyCode() == KeyEvent.VK_0) || (keyEvent.getKeyCode() == KeyEvent.VK_5))
        	  {        	  
        		 fieldOfView = 0.5;
                 cam.setFieldOfView(fieldOfView);
                 cam.update(cam);
        		 sphereTransparency = 0.9;
        		 sphere.setTransparency(sphereTransparency);
        		 sphere.update(sphere);
                 disp.fit();
        	  };        	  
          }
       };
       disp.addKeyListener(internalKeyListener);       
   }; // End of init()
   
   
   public void addVertex(double x, double y, double z)
   {
      sop.addVertex(new PdVector(x, y, z));
	  sop.update(sop);         
   }
   
   public void reset()
   {
      disp.removeGeometry(sop);
      sop = new PgPointSet(3);
      sop.setGlobalVertexColor(Color.BLUE);
      sop.setGlobalVertexSize(0.5);
      sop.showVertexSizes(true);      
      disp.addGeometry(sop);
      disp.update(disp);
   }   
};
